@@ -544,6 +544,7 @@ t_signal *signal_new(int length, int nchans, t_float sr, t_sample *scalarptr)
     ret->s_nchans = nchans;
     ret->s_nalloc = allocsize;
     ret->s_sr = sr;
+    ret->s_overlap = 0;
     ret->s_refcount = 0;
     ret->s_borrowedfrom = 0;
     if (THIS->u_loud) post("new %lx: %lx", ret, ret->s_vec);
@@ -552,7 +553,9 @@ t_signal *signal_new(int length, int nchans, t_float sr, t_sample *scalarptr)
 
 t_signal *signal_newlike(const t_signal *sig)
 {
-    return (signal_new(sig->s_length, sig->s_nchans, sig->s_sr, 0));
+    t_signal *s = signal_new(sig->s_length, sig->s_nchans, sig->s_sr, 0);
+    s->s_overlap = sig->s_overlap;
+    return s;
 }
 
 void signal_setborrowed(t_signal *sig, t_signal *sig2)
@@ -565,6 +568,8 @@ void signal_setborrowed(t_signal *sig, t_signal *sig2)
     sig->s_vec = sig2->s_vec;
     sig->s_length = sig2->s_length;
     sig->s_nchans = sig2->s_nchans;
+    sig->s_sr = sig2->s_sr;
+    sig->s_overlap = sig2->s_overlap;
     sig->s_nalloc = sig2->s_nalloc;
     sig2->s_refcount++;
     if (THIS->u_loud) post("set borrowed %lx: from %lx vec %lx",
@@ -575,13 +580,15 @@ void signal_setborrowed(t_signal *sig, t_signal *sig2)
     on output signal - we assume it's currently a pointer to the null signal */
 void signal_setmultiout(t_signal **sig, int nchans)
 {
+    int overlap = (*sig)->s_overlap;
     *sig = signal_new((*sig)->s_length, nchans, (*sig)->s_sr, 0);
+    (*sig)->s_overlap = overlap;
 }
 
 static int signal_compatible(t_signal *s1, t_signal *s2)
 {
     return (s1->s_length == s2->s_length && s1->s_nchans == s2->s_nchans
-        && s1->s_sr == s2->s_sr);
+        && s1->s_sr == s2->s_sr && s1->s_overlap == s2->s_overlap);
 }
 
 /* ------------------ ugen ("unit generator") sorting ----------------- */
@@ -636,14 +643,17 @@ struct _dspcontext
 };
 #define DC_LENGTH(x) ((x)->dc_nullsignal.s_length)
 #define DC_SR(x) ((x)->dc_nullsignal.s_sr)
+#define DC_OVERLAP(x) ((x)->dc_nullsignal.s_overlap)
 
 #define t_dspcontext struct _dspcontext
 
     /* get a new signal for the current context - used by clone~ object */
 t_signal *signal_newfromcontext(int borrowed, int nchans)
 {
-    return (signal_new((borrowed? 0 : DC_LENGTH(THIS->u_context)), nchans,
-        DC_SR(THIS->u_context), 0));
+    t_signal *s = signal_new((borrowed? 0 : DC_LENGTH(THIS->u_context)), nchans,
+        DC_SR(THIS->u_context), 0);
+    s->s_overlap = DC_OVERLAP(THIS->u_context);
+    return s;
 }
 
 void ugen_stop(void)
@@ -885,6 +895,7 @@ static void ugen_doit(t_dspcontext *dc, t_ugenbox *u)
                 dsp_add_scalarcopy(scalar, uin->i_signal->s_vec,
                     uin->i_signal->s_n);
             }
+            uin->i_signal->s_overlap = DC_OVERLAP(dc);
             uin->i_signal->s_refcount = 1;
         }
     }
@@ -930,6 +941,8 @@ static void ugen_doit(t_dspcontext *dc, t_ugenbox *u)
         else if (flags & CLASS_MULTICHANNEL)
             *sig = &dc->dc_nullsignal;
         else *sig = signal_new(DC_LENGTH(dc), 1, DC_SR(dc), 0);
+        (*sig)->s_overlap = DC_OVERLAP(dc);
+
     }
         /* now call the DSP scheduling routine for the ugen.  This
         routine must fill in "borrowed" signal outputs in case it's either
@@ -1042,7 +1055,7 @@ void ugen_done_graph(t_dspcontext *dc)
     t_block *blk;
     t_dspcontext *parent_context = dc->dc_parentcontext;
     t_float parent_srate;
-    int parent_vecsize;
+    int parent_vecsize, parent_overlap;
     int period, frequency, phase, calcsize;
     t_float srate;
     int chainblockbegin;    /* DSP chain onset before block prolog code */
@@ -1050,8 +1063,9 @@ void ugen_done_graph(t_dspcontext *dc)
     int chainafterall;      /* and after signal outlet epilog */
     int reblock = 0, switched;
     int downsample = 1, upsample = 1;
-    /* debugging printout */
+    int totaloverlap;
 
+        /* debugging printout */
     if (THIS->u_loud)
     {
         post("ugen_done_graph...");
@@ -1085,12 +1099,15 @@ void ugen_done_graph(t_dspcontext *dc)
     {
         parent_srate = DC_SR(parent_context);
         parent_vecsize = DC_LENGTH(parent_context);
+        parent_overlap = DC_OVERLAP(parent_context);
     }
     else
     {
         parent_srate = sys_getsr();
         parent_vecsize = sys_getblksize();
+        parent_overlap = 1;
     }
+    totaloverlap = parent_overlap;
     if (blk)
     {
         int realoverlap;
@@ -1119,6 +1136,8 @@ void ugen_done_graph(t_dspcontext *dc)
                 (downsample != 1) || (upsample != 1))
                     reblock = 1;
         switched = blk->x_switched;
+
+        totaloverlap = parent_overlap * realoverlap;
     }
     else
     {
@@ -1134,6 +1153,7 @@ void ugen_done_graph(t_dspcontext *dc)
     dc->dc_switched = switched;
     dc->dc_nullsignal.s_sr = srate;
     dc->dc_nullsignal.s_length = calcsize;
+    dc->dc_nullsignal.s_overlap = totaloverlap;
     dc->dc_nullsignal.s_nchans = -1;    /* fake so we can sanity check */
 
         /* if we're reblocking or switched, we now have to create output
@@ -1150,8 +1170,9 @@ void ugen_done_graph(t_dspcontext *dc)
         {
             if ((*sigp)->s_isborrowed && !(*sigp)->s_borrowedfrom)
             {
-                signal_setborrowed(*sigp,
-                    signal_new(parent_vecsize, 1, parent_srate, 0));
+                t_signal *s = signal_new(parent_vecsize, 1, parent_srate, 0);
+                s->s_overlap = totaloverlap;
+                signal_setborrowed(*sigp, s);
 
                 if (THIS->u_loud) post("set %lx->%lx", *sigp,
                     (*sigp)->s_borrowedfrom);
@@ -1233,6 +1254,7 @@ void ugen_done_graph(t_dspcontext *dc)
             if ((*sigp)->s_isborrowed && !(*sigp)->s_borrowedfrom)
             {
                 t_signal *s3 = signal_new(parent_vecsize, 1, parent_srate, 0);
+                s3->s_overlap = totaloverlap;
                 signal_setborrowed(*sigp, s3);
                 dsp_add_zero(s3->s_vec, s3->s_n);
                 if (THIS->u_loud)
