@@ -10,17 +10,14 @@ behavior for "gobjs" appears at the end of this file.  */
 #include "m_imp.h"
 #include <string.h>
 
-#ifdef _WIN32
-# include <malloc.h> /* MSVC or mingw on windows */
-#elif defined(__linux__) || defined(__APPLE__) || defined(HAVE_ALLOCA_H)
-# include <alloca.h> /* linux, mac, mingw, cygwin */
-#endif
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#endif
+#include "m_private_utils.h"
 
-#ifdef _MSC_VER
-#define snprintf _snprintf
+#if defined(_MSC_VER)
+#define INLINE __forceinline
+#elif defined(__GNUC__)
+#define INLINE inline __attribute__((always_inline))
+#else
+#define INLINE inline
 #endif
 
 union inletunion
@@ -187,7 +184,7 @@ static void inlet_anything(t_inlet *x, t_symbol *s, int argc, t_atom *argv)
     {
         /* the "symto" field is undefined for signal inlets, so we don't
          attempt to translate the selector, just forward the original msg. */
-        
+
         if (x->i_symfrom == &s_signal)
             typedmess(x->i_dest, s, argc, argv);
         else
@@ -351,6 +348,7 @@ int backtracer_tracing;
 t_class *backtracer_class;
 
 static PERTHREAD int stackcount = 0; /* iteration counter */
+static PERTHREAD int overflow = 0;
 #define STACKITER 1000 /* maximum iterations allowed */
 
 static PERTHREAD int outlet_eventno;
@@ -359,11 +357,26 @@ static PERTHREAD int outlet_eventno;
     messages so that  the outlet functions can check to prevent stack overflow]
     from message recursion.  Also count message initiations. */
 
+static INLINE int stackcount_add(void)
+{
+        /* set overflow flag to prevent any further messaging */
+    if (++stackcount >= STACKITER)
+        overflow = 1;
+    return !overflow;
+}
+
+static INLINE void stackcount_release(void)
+{
+        /* once the stack is completely unwound, we can clear the overflow flag */
+    if (--stackcount == 0)
+        overflow = 0;
+}
+
 void outlet_setstacklim(void)
 {
     t_msgstack *m;
     while ((m = backtracer_stack))
-        backtracer_stack = m->m_next; t_freebytes(m, sizeof (*m));
+        backtracer_stack = m->m_next, t_freebytes(m, sizeof (*m));
     stackcount = 0;
     outlet_eventno++;
 }
@@ -555,19 +568,19 @@ static void outlet_stackerror(t_outlet *x)
 void outlet_bang(t_outlet *x)
 {
     t_outconnect *oc;
-    if(++stackcount >= STACKITER)
+    if(!stackcount_add())
         outlet_stackerror(x);
     else
-    for (oc = x->o_connections; oc; oc = oc->oc_next)
-        pd_bang(oc->oc_to);
-    --stackcount;
+        for (oc = x->o_connections; oc; oc = oc->oc_next)
+            pd_bang(oc->oc_to);
+    stackcount_release();
 }
 
 void outlet_pointer(t_outlet *x, t_gpointer *gp)
 {
     t_outconnect *oc;
     t_gpointer gpointer;
-    if(++stackcount >= STACKITER)
+    if(!stackcount_add())
         outlet_stackerror(x);
     else
     {
@@ -575,51 +588,51 @@ void outlet_pointer(t_outlet *x, t_gpointer *gp)
         for (oc = x->o_connections; oc; oc = oc->oc_next)
             pd_pointer(oc->oc_to, &gpointer);
     }
-    --stackcount;
+    stackcount_release();
 }
 
 void outlet_float(t_outlet *x, t_float f)
 {
     t_outconnect *oc;
-    if(++stackcount >= STACKITER)
+    if(!stackcount_add())
         outlet_stackerror(x);
     else
-    for (oc = x->o_connections; oc; oc = oc->oc_next)
-        pd_float(oc->oc_to, f);
-    --stackcount;
+        for (oc = x->o_connections; oc; oc = oc->oc_next)
+            pd_float(oc->oc_to, f);
+    stackcount_release();
 }
 
 void outlet_symbol(t_outlet *x, t_symbol *s)
 {
     t_outconnect *oc;
-    if(++stackcount >= STACKITER)
+    if(!stackcount_add())
         outlet_stackerror(x);
     else
-    for (oc = x->o_connections; oc; oc = oc->oc_next)
-        pd_symbol(oc->oc_to, s);
-    --stackcount;
+        for (oc = x->o_connections; oc; oc = oc->oc_next)
+            pd_symbol(oc->oc_to, s);
+    stackcount_release();
 }
 
 void outlet_list(t_outlet *x, t_symbol *s, int argc, t_atom *argv)
 {
     t_outconnect *oc;
-    if(++stackcount >= STACKITER)
+    if(!stackcount_add())
         outlet_stackerror(x);
     else
-    for (oc = x->o_connections; oc; oc = oc->oc_next)
-        pd_list(oc->oc_to, s, argc, argv);
-    --stackcount;
+        for (oc = x->o_connections; oc; oc = oc->oc_next)
+            pd_list(oc->oc_to, s, argc, argv);
+    stackcount_release();
 }
 
 void outlet_anything(t_outlet *x, t_symbol *s, int argc, t_atom *argv)
 {
     t_outconnect *oc;
-    if(++stackcount >= STACKITER)
+    if(!stackcount_add())
         outlet_stackerror(x);
     else
-    for (oc = x->o_connections; oc; oc = oc->oc_next)
-        typedmess(oc->oc_to, s, argc, argv);
-    --stackcount;
+        for (oc = x->o_connections; oc; oc = oc->oc_next)
+            typedmess(oc->oc_to, s, argc, argv);
+    stackcount_release();
 }
 
     /* get the outlet's declared symbol */
@@ -900,14 +913,22 @@ int obj_issignaloutlet(const t_object *x, int m)
     return (o2 && (o2->o_sym == &s_signal));
 }
 
+    /* return a pointer to a scalar holding the inlet's value.  If we
+    can't find a value, return a pointer to a fixed location holding zero.
+    This should only happen for a left-hand signal inlet for which no
+    "MAINSIGNALIN" has been provided, in which case the object won't
+    promote scalars correctly.  Nonetheless we provide it so that at least
+    such a badly written object won't crash Pd. */
 t_float *obj_findsignalscalar(const t_object *x, int m)
 {
     t_inlet *i;
+    static float obj_scalarzero = 0;
     if (x->ob_pd->c_firstin && x->ob_pd->c_floatsignalin)
     {
         if (!m--)
             return (x->ob_pd->c_floatsignalin > 0 ?
-                (t_float *)(((char *)x) + x->ob_pd->c_floatsignalin) : 0);
+                (t_float *)(((char *)x) + x->ob_pd->c_floatsignalin) :
+                    &obj_scalarzero);
     }
     for (i = x->ob_inlet; i; i = i->i_next)
         if (i->i_symfrom == &s_signal)
@@ -915,7 +936,7 @@ t_float *obj_findsignalscalar(const t_object *x, int m)
         if (m-- == 0)
             return (&i->i_un.iu_floatsignalvalue);
     }
-    return (0);
+    return (&obj_scalarzero);   /* this should never happen but OK wtf. */
 }
 
 /* and these are only used in g_io.c... */
